/* vim: set et sw=2 ts=2 sts=2 ff=unix fenc=utf8: */
/* Author: Binux<i@binux.me> */
/*         http://binux.me */
/* Created on 2014-02-23 00:28:30 */
/* vim: set et sw=2 ts=2 sts=2 ff=unix fenc=utf8: */
/* Author: Binux<i@binux.me> */
/*         http://binux.me */
/* Created on 2014-07-16 19:18:30 */
.projects {
  min-width: 850px;
  border-bottom: 1px solid #ddd;
}
.projects .project-group {
  width: 80px;
}
.projects .project-name {
  font-weight: bold;
}
.projects .project-status {
  width: 100px;
}
.projects .project-status > span {
  border: solid 1px #666666;
  padding: 1px 5px 0 5px;
  background: #808080;
  color: white;
}
.projects span.status-TODO {
  border: solid 1px #ec971f;
  padding: 1px 5px 0 5px;
  background: #f0ad4e;
  color: white;
}
.projects span.status-STOP {
  border: solid 1px #c9302c;
  padding: 1px 5px 0 5px;
  background: #d9534f;
  color: white;
}
.projects span.status-CHECKING {
  border: solid 1px #dcbe00;
  padding: 1px 5px 0 5px;
  background: #ffde10;
  color: white;
}
.projects span.status-DEBUG {
  border: solid 1px #3071a9;
  padding: 1px 5px 0 5px;
  background: #428bca;
  color: white;
}
.projects span.status-RUNNING {
  border: solid 1px #449d44;
  padding: 1px 5px 0 5px;
  background: #5cb85c;
  color: white;
}
.projects .project-rate {
  width: 110px;
}
.projects th.project-progress {
  position: relative;
}
.projects th.project-progress span {
  position: absolute;
}
.projects td.project-progress {
  position: relative;
  min-width: 5%;
}
.projects td.project-progress.progress-all {
  min-width: 10%;
}
.projects td.project-progress .progress {
  position: relative;
  margin: 0;
  background-color: #aaa;
}
.projects td.project-progress .progress .progress-text {
  width: 100%;
  text-align: center;
  position: absolute;
  font-weight: bold;
  color: #fff;
  pointer-events: none;
}
.projects td.project-progress .progress .progress-bar {
  -webkit-transition: none;
  transition: none;
}
.projects .project-actions {
  width: 200px;
}
.global-btn {
  margin-top: -5px;
  padding: 0 60px 10px 10px;
}
