/* vim: set et sw=2 ts=2 sts=2 ff=unix fenc=utf8: */
/* Author: Binux<i@binux.me> */
/*         http://binux.me */
/* Created on 2014-07-18 23:20:46 */
/* vim: set et sw=2 ts=2 sts=2 ff=unix fenc=utf8: */
/* Author: Binux<i@binux.me> */
/*         http://binux.me */
/* Created on 2014-07-16 19:18:30 */
/* vim: set et sw=2 ts=2 sts=2 ff=unix fenc=utf8: */
/* Author: Binux<i@binux.me> */
/*         http://binux.me */
/* Created on 2014-07-16 19:20:30 */
.base-info {
  padding: 10px 15px 2px 15px;
  background-color: #f5f5f5;
  border-bottom: 1px solid #ddd;
}
.more-info {
  padding: 10px 15px;
}
.more-info dd {
  display: block;
  font-family: monospace;
  white-space: pre;
  word-break: break-all;
  word-wrap: break-word;
  margin: 1em 0px;
}
.status-1 {
  border: solid 1px #3071a9;
  padding: 1px 5px 0 5px;
  background: #428bca;
  color: white;
}
.status-2 {
  border: solid 1px #449d44;
  padding: 1px 5px 0 5px;
  background: #5cb85c;
  color: white;
}
.status-3 {
  border: solid 1px #c9302c;
  padding: 1px 5px 0 5px;
  background: #d9534f;
  color: white;
}
.status-4 {
  border: solid 1px #666666;
  padding: 1px 5px 0 5px;
  background: #808080;
  color: white;
}
.url {
  font-size: 120%;
  text-decoration: underline;
}
.callback {
  color: #f0ad4e;
  font-weight: bold;
}
.callback:hover,
.callback:focus {
  color: #ec971f;
}
dt .glyphicon-ok {
  color: #5cb85c;
}
dt .glyphicon-remove {
  color: #d9534f;
}
.tasks {
  margin: 0;
  padding: 0;
  list-style-type: none;
}
.tasks li {
  padding: 10px 15px 2px 15px;
  background-color: #f5f5f5;
  border-bottom: 1px solid #ddd;
}
.tasks li:nth-child(even) {
  background-color: white;
}
.tasks .url {
  display: inline-block;
  vertical-align: bottom;
  max-width: 40em;
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}
.tasks .update-time {
  font-weight: bold;
}
