/* vim: set et sw=2 ts=2 sts=2 ff=unix fenc=utf8: */
/* Author: Binux<i@binux.me> */
/*         http://binux.me */
/* Created on 2014-02-23 00:28:30 */
/* vim: set et sw=2 ts=2 sts=2 ff=unix fenc=utf8: */
/* Author: Binux<i@binux.me> */
/*         http://binux.me */
/* Created on 2014-07-16 19:18:30 */
body {
  margin: 0;
  padding: 0;
  height: 100%;
  overflow: hidden;
}
.warning {
  color: #f0ad4e;
}
.error {
  color: #d9534f;
}
#control {
  z-index: 9999;
  min-width: 760px;
  width: 100%;
  height: 35px;
  position: fixed;
  left: 0;
  right: 0;
  background-color: #eeeeee;
  box-shadow: 0px 1px 2px #999999;
}
#control div {
  line-height: 35px;
  margin-left: 10px;
  margin-right: 10px;
}
#editarea {
  width: 100%;
  position: fixed;
  top: 37px;
  left: 0;
  right: 0;
  bottom: 0;
}
.debug-panel {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
}
.resize {
  background-color: #555555;
  cursor: ew-resize;
}
.resize:hover + .debug-panel {
  border-left: dashed 1px #555555 !important;
}
.overlay {
  position: absolute;
  top: 0;
  bottom: 0;
  left: 0;
  right: 0;
  z-index: 9999;
  background: rgba(0, 0, 0, 0.4);
}
.focus .CodeMirror-activeline-background {
  background: #e8f2ff !important;
}
.CodeMirror-activeline-background {
  background: transparent !important;
}
#task-panel {
  height: 100%;
  overflow-x: auto;
}
#run-task-btn {
  z-index: 99;
  position: absolute;
  top: 0;
  right: 0;
  background: #5cb85c;
  border-radius: 0 0 0 5px;
  color: white;
  margin: 0;
  padding: 3px 7px 5px 10px;
  cursor: pointer;
  font-weight: bold;
  line-height: 15px;
}
#run-task-btn:hover {
  background: #449d44;
}
#undo-redo-btn-group {
  z-index: 99;
  position: absolute;
  top: 0;
  right: 0;
  background: #91cf91;
  border-radius: 0 0 0 5px;
  color: white;
  margin: 0;
  padding: 3px 7px 5px 10px;
  cursor: pointer;
  font-weight: bold;
  line-height: 15px;
  top: auto;
  bottom: 0;
  border-radius: 5px 0 0 0;
  padding: 5px 0 3px 0;
  box-shadow: 0px 0px 30px #91cf91;
  overflow: hidden;
}
#undo-redo-btn-group:hover {
  background: #6ec06e;
}
#undo-redo-btn-group:hover {
  background: #91cf91;
}
#undo-redo-btn-group a {
  color: white;
  text-decoration: none;
  padding: 5px 7px 3px 10px;
}
#undo-redo-btn-group a:hover {
  background: #6ec06e;
}
#save-task-btn {
  z-index: 99;
  position: absolute;
  top: 0;
  right: 0;
  background: #428bca;
  border-radius: 0 0 0 5px;
  color: white;
  margin: 0;
  padding: 3px 7px 5px 10px;
  cursor: pointer;
  font-weight: bold;
  line-height: 15px;
}
#save-task-btn:hover {
  background: #3071a9;
}
#task-editor {
  position: relative;
}
#task-editor .CodeMirror {
  height: auto;
  padding-bottom: 3px;
  background: #c7e6c7;
}
#task-editor .CodeMirror-scroll {
  overflow-x: auto;
  overflow-y: hidden;
}
#task-editor.focus .CodeMirror-activeline-background {
  background: #eaf6ea !important;
}
#tab-control {
  list-style-type: none;
  position: absolute;
  bottom: 0;
  right: 0;
  margin: 8px 20px;
  padding: 0;
}
#tab-control li {
  position: relative;
  float: right;
  padding: 1px 7px 0 7px;
  line-height: 21px;
  margin-left: 10px;
  border-radius: 5px;
  border: solid 1px #428bca;
  background: white;
  color: #428bca;
  cursor: pointer;
}
#tab-control li:hover {
  background: #6aa3d5;
  color: white;
}
#tab-control li.active {
  background: #428bca;
  color: white;
}
#tab-control li span {
  position: absolute;
  top: -5px;
  right: -10px;
  background: #d9534f;
  color: white;
  font-size: 80%;
  font-weight: bold;
  padding: 2px 5px 0 5px;
  border-radius: 10px;
}
#debug-tabs {
  position: relative;
  margin-bottom: 45px;
}
#tab-web iframe {
  border-width: 0;
  width: 100%;
}
#tab-html {
  margin: 0;
  padding: 7px 5px;
}
#tab-html pre {
  margin: 0;
  padding: 0;
}
#tab-follows .newtask {
  position: relative;
  height: 30px;
  line-height: 30px;
  background: #fceedb;
  border-bottom: solid 1px #f0ad4e;
  border-top: solid 1px #f0ad4e;
  margin-top: -1px;
  padding-left: 5px;
  padding-right: 70px;
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  cursor: pointer;
}
#tab-follows .newtask:hover {
  background: #f8d9ac;
}
#tab-follows .newtask:hover .task-more {
  background: #f8d9ac;
}
#tab-follows .newtask .task-callback {
  color: #ec971f;
}
#tab-follows .newtask .task-url {
  font-size: 95%;
  text-decoration: underline;
  font-weight: lighter;
  color: #428bca;
}
#tab-follows .newtask .task-more {
  position: absolute;
  right: 33px;
  top: 0px;
  float: right;
  color: #f0ad4e;
  padding: 0 10px;
  background: #fceedb;
  border-radius: 10px;
}
#tab-follows .newtask .task-run {
  position: absolute;
  right: 0px;
  top: 0px;
  font-size: 80%;
  padding: 0 10px 0 30px;
  float: right;
  border-bottom: solid 1px #a3d7a3;
  border-top: solid 1px #a3d7a3;
  background: #80c780;
  color: white;
  text-shadow: 0 0 10px white;
  font-weight: bold;
}
#tab-follows .newtask .task-run:hover {
  background: #5cb85c;
}
#tab-follows .task-show pre {
  margin: 5px 5px 10px 5px;
}
#python-editor {
  position: absolute;
  top: 0;
  width: 100%;
  bottom: 0;
}
#python-editor .CodeMirror {
  height: 100%;
  padding-bottom: 20px;
}
#python-log {
  z-index: 88;
  position: absolute;
  bottom: 0;
  width: 100%;
  min-height: 10px;
  max-height: 40%;
  background: rgba(0, 0, 0, 0.6);
  overflow: auto;
}
#python-log #python-log-show {
  z-index: 89;
  width: auto;
  padding-top: 5px;
  background: #d9534f;
  box-shadow: 0px 2px 20px #d9534f;
  cursor: pointer;
}
#python-log pre {
  margin: 0;
  padding: 10px 10px;
  color: white;
}
